package org.jenkinsci.plugins.p4_client.review;

import hudson.model.Action;
import hudson.model.ParameterValue;
import hudson.model.AbstractProject;
import hudson.model.Cause;
import hudson.model.CauseAction;
import hudson.model.ParameterDefinition;
import hudson.model.ParametersAction;
import hudson.model.StringParameterDefinition;
import hudson.model.StringParameterValue;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.servlet.ServletException;

import jenkins.model.Jenkins;
import jenkins.util.TimeDuration;
import net.sf.json.JSONObject;

import org.kohsuke.stapler.StaplerRequest;
import org.kohsuke.stapler.StaplerResponse;

public class ReviewAction implements Action {

	private final AbstractProject<?, ?> project;

	public AbstractProject<?, ?> getProject() {
		return project;
	}

	public ReviewAction(AbstractProject<?, ?> project) {
		this.project = project;
	}

	public String getIconFileName() {
		return "/plugin/p4-client/icons/p4.png";
	}

	public String getDisplayName() {
		return "Build Review";
	}

	public String getUrlName() {
		return "review";
	}

	/**
	 * Jelly Method
	 * 
	 * @return
	 */
	public List<StringParameterValue> getAvailableParameters() {
		List<StringParameterValue> stringParameters = new ArrayList<StringParameterValue>();

		for (ParameterDefinition parameterDefinition : getParameterDefinitions()) {
			StringParameterValue stringParameter = new StringParameterValue(
					parameterDefinition.getName(),
					parameterDefinition.getDescription());
			stringParameters.add(stringParameter);
		}

		return stringParameters;
	}

	public void doBuildSubmit(StaplerRequest req, StaplerResponse rsp)
			throws IOException, ServletException {

		project.checkPermission(AbstractProject.BUILD);

		List<ParameterValue> values = new ArrayList<ParameterValue>();

		JSONObject formData = req.getSubmittedForm();
		if (!formData.isEmpty()) {
			for (ParameterDefinition parameterDefinition : getParameterDefinitions()) {
				ParameterValue parameterValue = parameterDefinition
						.createValue(req);

				// This will throw an exception if the provided value is not a
				// valid option for the parameter.
				// This is the desired behavior, as we want to ensure valid
				// submissions.
				values.add(parameterValue);
			}
		}

		TimeDuration delay = new TimeDuration(project.getQuietPeriod());

		Jenkins.getInstance()
				.getQueue()
				.schedule(project, delay.getTime(),
						new ParametersAction(values),
						new CauseAction(new Cause.UserIdCause()));

		rsp.sendRedirect("../");
	}

	public void doBuild(StaplerRequest req, StaplerResponse rsp)
			throws IOException, ServletException {

		project.checkPermission(AbstractProject.BUILD);

		List<ParameterValue> values = new ArrayList<ParameterValue>();
		List<ParameterDefinition> defs = getParameterDefinitions();
		for (ParameterDefinition d : defs) {
			ParameterValue value = d.createValue(req);
			if (value != null) {
				values.add(value);
			}
		}

		TimeDuration delay = new TimeDuration(project.getQuietPeriod());

		Jenkins.getInstance()
				.getQueue()
				.schedule(project, delay.getTime(),
						new ParametersAction(values),
						new CauseAction(new Cause.UserIdCause()));

		// send the user back to the job top page.
		rsp.sendRedirect(".");
	}

	private List<ParameterDefinition> getParameterDefinitions() {
		List<ParameterDefinition> swarm = new ArrayList<ParameterDefinition>();
		swarm.add(new StringParameterDefinition("review", "0"));
		swarm.add(new StringParameterDefinition("change", "0"));
		swarm.add(new StringParameterDefinition("status", null));
		swarm.add(new StringParameterDefinition("pass", null));
		swarm.add(new StringParameterDefinition("fail", null));

		return swarm;
	}

	final static Logger LOGGER = Logger.getLogger(ReviewAction.class.getName());
}
